@@ -62,8 +62,8 @@ int        delay_counter     = 0;
 int        wait_bonus        = 0;
 
 int        session_flag      = 0;
-int        session_xpos      = 0;
-int        session_ypos      = 0;
+int        session_xpos      = -1;
+int        session_ypos      = -1;
 int        session_position  = 0;
 
 /*
@@ -111,46 +111,31 @@ static gint    keyboard_cb(GtkWidget*, GdkEventKey*, gpointer);
 static void    update_score_label();
 static void    update_teleports_label();
 static void    update_level_label();
-static error_t parse_an_arg (int key, char *arg, struct argp_state *state);
 static void    show_teleport_message(int safe);
        void    show_rollover_message();
 
-/* The command-line options we understand.  */
-static struct argp_option options[] =
-{
-    { "scenario", 's', N_("NAME"), 0, N_("Set game scenario"), 1 },
-    { NULL, 'x', N_("X"), OPTION_HIDDEN, NULL, 1 },
-    { NULL, 'y', N_("Y"), OPTION_HIDDEN, NULL, 1 },    
-    { NULL, 0, NULL, 0, NULL, 0 }
-};
-
-/* How to parse the command-line options.  */
-static struct argp parser =
-{
-    options,
-    parse_an_arg,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL
+static struct poptOption options[] = {
+  {"scenario", 's', POPT_ARG_STRING, &cmdline_scenario, 0, N_("Set game scenario"), N_("NAME")},
+  {NULL, 'x', POPT_ARG_INT, &session_xpos, 0, NULL, N_("X")},
+  {NULL, 'y', POPT_ARG_INT, &session_ypos, 0, NULL, N_("Y")},
+  {NULL, '\0', 0, NULL, 0}
 };
 
 /*
  * Game menu entries
  */
 GnomeUIInfo gamemenu[] = {
-    {GNOME_APP_UI_ITEM, N_("New"), NULL, new_cb, NULL, NULL,
-        GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 0, 0, NULL},
+    {GNOME_APP_UI_ITEM, N_("_New"), NULL, new_cb, NULL, NULL,
+        GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 'n', GDK_CONTROL_MASK, NULL},
 
-    {GNOME_APP_UI_ITEM, N_("Properties..."), NULL, properties_cb, NULL, NULL,
+    {GNOME_APP_UI_ITEM, N_("_Properties..."), NULL, properties_cb, NULL, NULL,
         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, 0, NULL},
 
-    {GNOME_APP_UI_ITEM, N_("Scores..."), NULL, game_scores_cb, NULL, NULL,
+    {GNOME_APP_UI_ITEM, N_("_Scores"), NULL, game_scores_cb, NULL, NULL,
         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SCORES, 0, 0, NULL},
 
-    {GNOME_APP_UI_ITEM, N_("Exit"), NULL, quit_cb, NULL, NULL,
-        GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 0, 0, NULL},
+    {GNOME_APP_UI_ITEM, N_("E_xit"), NULL, quit_cb, NULL, NULL,
+        GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'q', GDK_CONTROL_MASK, NULL},
 
     {GNOME_APP_UI_ENDOFINFO}
 };
@@ -159,7 +144,7 @@ GnomeUIInfo gamemenu[] = {
  * Help menu entries
  */
 GnomeUIInfo helpmenu[] = {
-    {GNOME_APP_UI_ITEM, N_("About..."), NULL, about_cb, NULL, NULL,
+    {GNOME_APP_UI_ITEM, N_("_About Gnobots"), NULL, about_cb, NULL, NULL,
         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL},
 
     {GNOME_APP_UI_HELP, NULL, NULL, "gnobots", NULL, NULL,
@@ -172,10 +157,10 @@ GnomeUIInfo helpmenu[] = {
  * Main menu
  */
 GnomeUIInfo mainmenu[] = {
-    {GNOME_APP_UI_SUBTREE, N_("Game"), NULL, gamemenu, NULL, NULL,
+    {GNOME_APP_UI_SUBTREE, N_("_Game"), NULL, gamemenu, NULL, NULL,
         GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
-    {GNOME_APP_UI_SUBTREE, N_("Help"), NULL, helpmenu, NULL, NULL,
+    {GNOME_APP_UI_SUBTREE, N_("_Help"), NULL, helpmenu, NULL, NULL,
         GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
         
     {GNOME_APP_UI_ENDOFINFO}
@@ -677,7 +662,7 @@ static void really_new_cb(
 GtkWidget *widget,
 gpointer  data
 ){
-    int button = (int)data;
+    gint button = GPOINTER_TO_INT (data);
     
     if(button != 0) return;
     
@@ -709,8 +694,9 @@ gpointer  data
                                      GNOME_STOCK_BUTTON_YES,
                                      GNOME_STOCK_BUTTON_NO,
                                      NULL);
+	gnome_dialog_set_parent (GNOME_DIALOG(box), GTK_WINDOW(app));
         gnome_dialog_set_default (GNOME_DIALOG(box), 0);
-        gnome_dialog_set_modal (GNOME_DIALOG(box));
+        gtk_window_set_modal (GTK_WINDOW(box), TRUE);
         gtk_signal_connect (GTK_OBJECT(box), "clicked",
                            GTK_SIGNAL_FUNC(really_new_cb), NULL);
         gtk_widget_show(box);
@@ -734,8 +720,9 @@ gpointer  data
                                      GNOME_STOCK_BUTTON_YES,
                                      GNOME_STOCK_BUTTON_NO,
                                      NULL);
+	gnome_dialog_set_parent (GNOME_DIALOG(box), GTK_WINDOW(app));
         gnome_dialog_set_default (GNOME_DIALOG(box), 0);
-        gnome_dialog_set_modal (GNOME_DIALOG(box));
+        gtk_window_set_modal (GTK_WINDOW(box), TRUE);
         gtk_signal_connect (GTK_OBJECT(box), "clicked",
                            GTK_SIGNAL_FUNC(really_quit_cb), NULL);
         gtk_widget_show(box);
@@ -749,7 +736,7 @@ static void really_quit_cb(
 GtkWidget   *widget,
 gpointer    data
 ){
-    int button = (int)data;
+    gint button = GPOINTER_TO_INT (data);
     
     if(button != 0) return;
     
@@ -757,7 +744,7 @@ gpointer    data
     
     save_properties();
     
-    gtk_widget_destroy(app);
+//    gtk_widget_destroy(app);
     gtk_main_quit();
 }
 
@@ -780,6 +767,8 @@ gpointer data
                              (const char **) authors,
                              _("Gnome Robots game"),
                              NULL);
+    gnome_dialog_set_parent(GNOME_DIALOG(about), GTK_WINDOW(app));
+    gtk_window_set_modal(GTK_WINDOW(about), TRUE);
     gtk_widget_show(about);
 }
 
@@ -1212,36 +1201,6 @@ static void update_level_label(
     gtk_label_set(GTK_LABEL(level_label), buffer);
 }
 
-/*
- * Command line parsing
- */
-static error_t parse_an_arg(
-int key, 
-char *arg, 
-struct argp_state *state
-){
-    switch(key) {
-        case 's':
-            cmdline_scenario = g_strdup(arg);
-            break;
-        case 'x':
-            session_flag |= 1;
-            session_xpos = atoi(arg);
-            break;
-        case 'y':
-            session_flag |= 2;
-            session_ypos = atoi(arg);
-            break;
-        case ARGP_KEY_SUCCESS:
-            if(session_flag == 3) session_position = 1;
-            break;
-        default:
-            return ARGP_ERR_UNKNOWN;
-    }
-
-    return 0;
-}
-
 /*
  * Show message when we cannot teleport
  */
@@ -1261,7 +1220,7 @@ int safe
 								GNOME_MESSAGE_BOX_INFO, 
 								GNOME_STOCK_BUTTON_OK, NULL);
 	}
-	gnome_dialog_set_modal(GNOME_DIALOG(box));
+	gtk_window_set_modal(GTK_WINDOW(box), TRUE);
 
 	gtk_widget_show(box);
 }
@@ -1277,7 +1236,7 @@ void show_rollover_message(
 _("Congratulations, you have defeated the Robots!\nBut can you do it again?"), 
 								GNOME_MESSAGE_BOX_INFO, 
 								GNOME_STOCK_BUTTON_OK, NULL);
-	gnome_dialog_set_modal(GNOME_DIALOG(box));
+	gtk_window_set_modal(GTK_WINDOW(box), TRUE);
 
 	gtk_widget_show(box);
 }
@@ -1289,28 +1248,28 @@ int main(
 int argc,
 char *argv[]
 ){
-    GtkWidget *box;
     GtkWidget *sttbl;
     GtkWidget *label;
+    GtkWidget *appbar;
     GnomeClient *client;
     char buf[PATH_MAX];
     
     srandom(time(NULL));
     
-    argp_program_version = VERSION;
-
     bindtextdomain(PACKAGE, GNOMELOCALEDIR);
     textdomain(PACKAGE);
 
     gnome_score_init("gnobots");
 
     
-    client = gnome_client_new_default();
+    gnome_init_with_popt_table("gnobots", VERSION, argc, argv, options, 0, NULL);
+
+    client = gnome_master_client();
 
     getcwd(buf, sizeof(buf));
 
     /* This doesn't seem to work properly ?? */
-    /* gnome_client_set_current_directory(client, buf); */
+    gnome_client_set_current_directory(client, buf);
 
     gtk_object_ref(GTK_OBJECT(client));
     gtk_object_sink(GTK_OBJECT(client));
@@ -1321,8 +1280,6 @@ char *argv[]
                        GTK_SIGNAL_FUNC (session_die), argv[0]);
 
     
-    gnome_init("gnobots", &parser, argc, argv, 0, NULL);
-
     app = gnome_app_new("gnobots", _("Gnome Robots") );
     /* no resize allowed */
     gtk_window_set_policy(GTK_WINDOW(app), FALSE, FALSE, TRUE);
@@ -1334,22 +1291,20 @@ char *argv[]
     gnome_app_create_menus(GNOME_APP(app), mainmenu);
 
     /* move Help to the right-hand side */
-    gtk_menu_item_right_justify(GTK_MENU_ITEM(mainmenu[1].widget));
+/*    gtk_menu_item_right_justify(GTK_MENU_ITEM(mainmenu[1].widget));*/
 
     load_properties();
 
-    box = gtk_vbox_new(FALSE, 0);
-    gnome_app_set_contents(GNOME_APP(app), box);
 
     game_area = gtk_drawing_area_new();
-    gtk_box_pack_start(GTK_BOX(box), game_area, TRUE, TRUE, 0);
+    gnome_app_set_contents(GNOME_APP(app), game_area);
     gtk_widget_realize(game_area);
     gtk_drawing_area_size(GTK_DRAWING_AREA(game_area),
             TILE_WIDTH*GAME_WIDTH, TILE_HEIGHT*GAME_HEIGHT);
     gtk_widget_show(game_area);
 
+
     sttbl = gtk_table_new(1, 8, FALSE);
-    gtk_box_pack_start(GTK_BOX(box), sttbl, TRUE, TRUE, 0);
 
     label = gtk_label_new(_("Score:"));
     gtk_table_attach(GTK_TABLE(sttbl), label, 0, 1, 0, 1, 0, 0, 3, 3);
@@ -1380,8 +1335,11 @@ char *argv[]
     gtk_widget_show(level_label);
     
     gtk_widget_show(sttbl);
+
+    appbar = gnome_appbar_new(FALSE, FALSE, FALSE);
+    gtk_box_pack_start(GTK_BOX(appbar), sttbl, FALSE, FALSE, 0);
     
-    gtk_widget_show(box);
+    gnome_app_set_statusbar(GNOME_APP(app), appbar);
 
     load_bubble_pixmaps();
 
@@ -1392,7 +1350,7 @@ char *argv[]
     }
 
     /* Set the window position if it was set by the session manager */
-    if(session_position){
+    if(session_xpos >= 0 && session_ypos >= 0){
         gtk_widget_set_uposition(app, session_xpos, session_ypos);
     }
     
