@@ -1,6 +1,6 @@
 /* gnome-stones - preferences.h
  *
- * Time-stamp: <1998/08/29 14:44:48 carsten>
+ * Time-stamp: <1998/12/06 17:33:09 carsten>
  *
  * Copyright (C) 1998 Carsten Schaar
  *
@@ -202,7 +202,6 @@ _preferences_save (GnomeClient        *client,
   else
     gnome_config_push_prefix (gnome_client_get_config_prefix (client));
 
-
   gnome_config_clean_section ("Preferences");
 
 
@@ -216,7 +215,6 @@ _preferences_save (GnomeClient        *client,
 	  break;
 	}
     }
-
   
   if (devicename)
     gnome_config_set_string ("Preferences/Joystick device",  devicename);
@@ -262,7 +260,6 @@ preferences_restore (void)
 
   gnome_config_push_prefix (gnome_client_get_config_prefix (client));
 
-
   devicename= gnome_config_get_string ("Preferences/Joystick device=");
   if (devicename)
     {
@@ -432,6 +429,8 @@ preferences_dialog_new (void)
   GtkWidget *box;
   GtkWidget *label;
   GtkWidget *list;
+  GtkWidget *scrolled;
+
   PreferencesData *prdata;
 
   prdata= g_malloc (sizeof (PreferencesData));
@@ -443,7 +442,8 @@ preferences_dialog_new (void)
 			_("Gnome-Stones Preferences"));
 
   /* The first page of our preferences dialog. */
-  box= gtk_vbox_new (TRUE, 4);
+  box= gtk_vbox_new (FALSE, GNOME_PAD);
+  gtk_container_set_border_width (GTK_CONTAINER (box), GNOME_PAD_SMALL);
 
   /* The list of game names.  */
   list= gtk_clist_new (3);
@@ -459,12 +459,14 @@ preferences_dialog_new (void)
   gtk_clist_column_titles_show (GTK_CLIST (list));
   gtk_widget_set_usize (list, -2, 200);
   
-  gtk_clist_set_policy (GTK_CLIST (list), 
-			GTK_POLICY_ALWAYS, 
-			GTK_POLICY_AUTOMATIC);
   gtk_clist_set_selection_mode (GTK_CLIST (list), GTK_SELECTION_SINGLE);
-  
-  gtk_box_pack_start (GTK_BOX (box), list, FALSE, FALSE, 0);
+
+  scrolled = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
+                                  GTK_POLICY_ALWAYS,
+                                  GTK_POLICY_AUTOMATIC);
+  gtk_container_add (GTK_CONTAINER (scrolled), list);
+  gtk_box_pack_start (GTK_BOX (box), scrolled, FALSE, FALSE, GNOME_PAD_SMALL);
 
   gtk_clist_freeze (GTK_CLIST (list));
   prdata->selected_game= -1;
@@ -500,6 +502,7 @@ preferences_dialog_new (void)
 		      prdata);
 
   gtk_widget_show (list);
+  gtk_widget_show (scrolled);
   gtk_widget_show (box);
 
   label= gtk_label_new (_("Game"));
@@ -512,7 +515,8 @@ preferences_dialog_new (void)
   prdata->joystick_deviceid    = joystick_deviceid;
   prdata->joystick_switch_level= joystick_switch_level;
 
-  box= gtk_vbox_new (TRUE, 4);
+  box= gtk_vbox_new (FALSE, GNOME_PAD);
+  gtk_container_set_border_width (GTK_CONTAINER (box), GNOME_PAD_SMALL);
   
   {
     guint      i;
@@ -527,19 +531,19 @@ preferences_dialog_new (void)
     GList     *devices;
 
     frame= gtk_frame_new (_("Device"));
-    gtk_box_pack_start (GTK_BOX (box), frame, TRUE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (box), frame, FALSE, FALSE, GNOME_PAD_SMALL);
     gtk_widget_show (frame);
 
-    vbox= gtk_vbox_new (FALSE, 2);
+    vbox= gtk_vbox_new (FALSE, GNOME_PAD);
     gtk_container_add (GTK_CONTAINER (frame), vbox);
     gtk_widget_show (vbox);
 
-    hbox= gtk_hbox_new (FALSE, 2);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
+    hbox= gtk_hbox_new (FALSE, GNOME_PAD);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, GNOME_PAD_SMALL);
     gtk_widget_show (hbox);
 
     label= gtk_label_new (_("Joystick device:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
+    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
     gtk_widget_show (label);
     
     device_menu= gtk_menu_new ();
@@ -584,20 +588,20 @@ preferences_dialog_new (void)
     gtk_widget_show (hbox);
     gtk_widget_show (optionmenu);
 
-    frame= gtk_frame_new (_("Binary joytick emulation"));
-    gtk_box_pack_start (GTK_BOX (box), frame, TRUE, FALSE, 0);
+    frame= gtk_frame_new (_("Digital joytick emulation"));
+    gtk_box_pack_start (GTK_BOX (box), frame, FALSE, FALSE, GNOME_PAD_SMALL);
     gtk_widget_show (frame);
 
-    vbox= gtk_vbox_new (FALSE, 2);
+    vbox= gtk_vbox_new (FALSE, GNOME_PAD);
     gtk_container_add (GTK_CONTAINER (frame), vbox);
     gtk_widget_show (vbox);
 
-    hbox= gtk_hbox_new (FALSE, 2);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
+    hbox= gtk_hbox_new (FALSE, GNOME_PAD);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, GNOME_PAD_SMALL);
     gtk_widget_show (hbox);
 
     label= gtk_label_new (_("Switch level:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
+    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
     gtk_widget_show (label);
 
     adjust= gtk_adjustment_new (prdata->joystick_switch_level,
@@ -609,7 +613,7 @@ preferences_dialog_new (void)
     
     scale= gtk_hscale_new (GTK_ADJUSTMENT (adjust));
     gtk_scale_set_digits (GTK_SCALE (scale), 2);
-    gtk_box_pack_start (GTK_BOX (hbox), scale, FALSE, FALSE, 2);
+    gtk_box_pack_start (GTK_BOX (hbox), scale, FALSE, FALSE, GNOME_PAD_SMALL);
     gtk_widget_show (scale);
     
     if (prdata->joystick_deviceid == GDK_CORE_POINTER)
@@ -628,10 +632,11 @@ preferences_dialog_new (void)
 			    box, label);
 
   /* The third page of our preferences dialog. */
-  box= gtk_vbox_new (TRUE, 4);
+  box= gtk_vbox_new (FALSE, GNOME_PAD);
+  gtk_container_set_border_width (GTK_CONTAINER (box), GNOME_PAD_SMALL);
 
   label= gtk_label_new (_("Not yet implemented!"));
-  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box), label, TRUE, FALSE, GNOME_PAD_SMALL);
 
   gtk_widget_show (label);
   gtk_widget_show (box);
@@ -662,6 +667,25 @@ gstones_exit (GnomeClient *client, gpointer client_data)
 }
 
 
+void
+discard_session_file (const gchar *file)
+{
+  gchar *file2= g_strdup (file);
+  gint   len  = strlen (file2);
+  
+  if (len && (file2[len-1]== '/'))
+    file2[len-1]= '\0';
+
+  gnome_config_push_prefix (file);
+  (void) gnome_config_get_int ("Dummy/Dummy=0");
+  gnome_config_pop_prefix ();
+  
+  /* All session information is written to the config file.  */
+  gnome_config_clean_file (file2);
+  gnome_config_sync ();
+}
+
+
 void
 session_management_init (void)
 {
@@ -672,6 +696,16 @@ session_management_init (void)
 		      GINT_TO_POINTER (FALSE));
   gtk_signal_connect (GTK_OBJECT (client), "die",
 		      GTK_SIGNAL_FUNC (gstones_exit), NULL);
+
+  if (GNOME_CLIENT_CONNECTED (client))
+    {
+      gchar *discard_command[3];
+      
+      discard_command[0]= program_invocation_name;
+      discard_command[1]= "--discard-file";
+      discard_command[2]= gnome_client_get_config_prefix (client);
+      gnome_client_set_discard_command (client, 3, discard_command);
+    }  
 }
 
 
