@@ -79,7 +79,7 @@ static GtkWidget *dicePixmaps[NUMBER_OF_DICE][NUMBER_OF_PIXMAPS];
 static GtkWidget *window, *status_bar, *diceBox[NUMBER_OF_DICE];
 static GtkWidget *diceTable;
 
-static gint gnome_modify_dice( GtkWidget *widget, GdkEvent *event,
+static gint gnome_modify_dice( GtkWidget *widget, GdkEventButton *event,
                                gpointer data );
 static gint gnome_roll_dice( GtkWidget *widget, GdkEvent *event,
                              gpointer data );
@@ -93,7 +93,7 @@ YahtzeeIdle()
 }
 
 
-void
+static void
 CheerWinner(void)
 {
         int winner;
@@ -124,7 +124,6 @@ NextPlayer(void)
                 CheerWinner();
                 return;
         } 
-
         NumberOfRolls = 0;
 
 	ShowoffPlayer(GTK_CLIST(ScoreList),CurrentPlayer,0);
@@ -144,6 +143,18 @@ NextPlayer(void)
                 }
         }
 
+#if 0
+	/* This is the old way.  I disapprove. */
+	if (HumansAreDone())      /* Because of multiple yahtzees, humans */
+	       NextPlayer();     /* may finish before computers */
+#else
+	/* If player 2 had more yahtzees than player 1, player 1 would be asked
+	 * to roll when he had no moves left, and player 2 would never get a
+	 * chance to roll, thus the game deadlocked. -- pschwan@cmu.edu */
+	if (players[CurrentPlayer].finished)
+	  NextPlayer();
+#endif
+
         SelectAllDice();
         RollSelectedDice();
 
@@ -152,9 +163,6 @@ NextPlayer(void)
                 NextPlayer();
                 return;
         }
-
-	if (HumansAreDone())      /* Because of multiple yahtzees, humans */
-                NextPlayer();     /* may finish before computers */
 }
 
 void
@@ -205,7 +213,7 @@ ShowPlayer(int num, int field)
 
 }
 
-gint 
+static gint 
 quit_game(GtkWidget *widget, gpointer data)
 {
         gtk_widget_destroy(window);
@@ -214,7 +222,7 @@ quit_game(GtkWidget *widget, gpointer data)
 }
 
 
-void 
+static void 
 GyahtzeeNewGame(void)
 {
         int i;
@@ -234,14 +242,14 @@ GyahtzeeNewGame(void)
 }
 
 
-gint 
+static gint 
 new_game_callback(GtkWidget *widget, gpointer data)
 {
         GyahtzeeNewGame();
 	return FALSE;
 }
 
-void
+static void
 UpdateDiePixmap(DiceInfo *d)
 {
         static int last_val[NUMBER_OF_DICE] = { 0 } ;
@@ -276,7 +284,7 @@ UpdateDie(int no)
 
 /* Callback on dice press */
 gint 
-gnome_modify_dice( GtkWidget *widget, GdkEvent *event, gpointer data ) 
+gnome_modify_dice( GtkWidget *widget, GdkEventButton *event, gpointer data ) 
 {
         DiceInfo *tmp = (DiceInfo *) data;
 
@@ -292,9 +300,34 @@ gnome_modify_dice( GtkWidget *widget, GdkEvent *event, gpointer data )
                 return TRUE;
         }
 
-        tmp->sel = 1 - tmp->sel;
-        
-        UpdateDiePixmap(tmp);
+#ifdef DEBUG
+	if (event->state & GDK_SHIFT_MASK)
+	  {
+	    /* If the user is holding down shift, cycle through the dice--very
+	     * useful for debugging. */
+	    tmp->val = (tmp->val + 1)%7;
+	    if (!tmp->val)
+	      tmp->val++;
+
+	    printf("val before update: %d\n", tmp->val);
+	    UpdateDiePixmap(tmp);
+	  }
+	else
+#endif
+	  {
+	    /* Stop play when player is marked finished */
+	    if (players[CurrentPlayer].finished)
+	      return TRUE;
+
+	    if (NumberOfRolls >= NUM_ROLLS) {
+	      say(_("You're only allowed three rolls! Select a score box."));
+	      return TRUE;
+	    }
+	    
+	    tmp->sel = 1 - tmp->sel;
+
+	    UpdateDiePixmap(tmp);
+	  }
 
 	return TRUE;
 }
@@ -323,7 +356,7 @@ say(char *fmt, ...)
 }
 
 
-gint
+static gint
 about(GtkWidget *widget, gpointer data)
 {
         GtkWidget *about;
@@ -348,14 +381,14 @@ ShowHighScores(void)
         gnome_scores_display (appName, appID, NULL, lastHighScore);
 }
 
-gint 
+static gint 
 score_callback(GtkWidget *widget, gpointer data)
 {
         ShowHighScores();
 	return FALSE;
 }
 
-gint 
+static gint 
 undo_callback(GtkWidget *widget, gpointer data)
 {
         int i;
@@ -370,17 +403,17 @@ undo_callback(GtkWidget *widget, gpointer data)
 
 /* Define menus later so we don't have to proto callbacks? */
 GnomeUIInfo gamemenu[] = {
-	{GNOME_APP_UI_ITEM, N_("New"), NULL, new_game_callback, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("_New"), NULL, new_game_callback, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 'N', GDK_CONTROL_MASK,
          NULL},
 
-	{GNOME_APP_UI_ITEM, N_("Properties..."), NULL, setup_game, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("_Properties..."), NULL, setup_game, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, 0, NULL},
 
-	{GNOME_APP_UI_ITEM, N_("Scores..."), NULL, score_callback, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("_Scores"), NULL, score_callback, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SCORES, 0, 0, NULL},
 
-	{GNOME_APP_UI_ITEM, N_("Exit"), NULL, quit_game, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("E_xit"), NULL, quit_game, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'Q', GDK_CONTROL_MASK,
          NULL},
 
@@ -389,10 +422,10 @@ GnomeUIInfo gamemenu[] = {
 };
 
 GnomeUIInfo editmenu[] = {
-	{GNOME_APP_UI_ITEM, N_("Properties..."), NULL, setup_game, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("_Properties..."), NULL, setup_game, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, 0, NULL},
 
-	{GNOME_APP_UI_ITEM, N_("Undo"), NULL, undo_callback, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("_Undo"), NULL, undo_callback, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO, 0, 0, NULL},
 
 	{GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL, NULL, NULL,
@@ -400,24 +433,26 @@ GnomeUIInfo editmenu[] = {
 };
 
 GnomeUIInfo helpmenu[] = {
+	{GNOME_APP_UI_ITEM, N_("_About Gyahtzee"), "Info about this program", about,
+	 NULL, NULL,
+	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL},
+
+#if 0
 	{GNOME_APP_UI_HELP, NULL, NULL, NULL, NULL, NULL,
-	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+	 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+#endif
 
-	{GNOME_APP_UI_ITEM, N_("About..."), NULL, about, NULL, NULL,
-	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL},
-
-	{GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL, NULL, NULL,
-	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL}
+	{GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL}
 };
 
 GnomeUIInfo mainmenu[] = {
-	{GNOME_APP_UI_SUBTREE, N_("Game"), NULL, gamemenu, NULL, NULL,
+	{GNOME_APP_UI_SUBTREE, N_("_Game"), NULL, gamemenu, NULL, NULL,
 	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
-	{GNOME_APP_UI_SUBTREE, N_("Edit"), NULL, editmenu, NULL, NULL,
+	{GNOME_APP_UI_SUBTREE, N_("_Edit"), NULL, editmenu, NULL, NULL,
 	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
-	{GNOME_APP_UI_SUBTREE, N_("Help"), NULL, helpmenu, NULL, NULL,
+	{GNOME_APP_UI_SUBTREE, N_("_Help"), NULL, helpmenu, NULL, NULL,
 	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
 	{GNOME_APP_UI_ENDOFINFO}
@@ -448,8 +483,8 @@ ExerciseAllPixmaps()
 #endif /* INCLUDE_UNUSED_FUNCS */
 
 
-void
-LoadDicePixmaps()
+static void
+LoadDicePixmaps(void)
 {
 	GtkWidget *tmp;
 	int i, j;
@@ -484,7 +519,7 @@ LoadDicePixmaps()
 
 
 static void
-GyahtzeeCreateMainWindow()
+GyahtzeeCreateMainWindow(void)
 {
         GtkWidget *all_boxes;
 	GtkWidget *mbutton;
@@ -501,7 +536,6 @@ GyahtzeeCreateMainWindow()
 
 	/*---- Menus ----*/
 	gnome_app_create_menus(GNOME_APP(window), mainmenu);
-        gtk_menu_item_right_justify(GTK_MENU_ITEM(mainmenu[2].widget));
 
 
 	/*---- Status Bar ----*/
@@ -583,7 +617,15 @@ main (int argc, char *argv[])
         YahtzeeInit();
 
 	/* Create gnome client */
-        gnome_init(appID, &GyahtzeeParser, argc, argv, 0, NULL);
+        gnome_init_with_popt_table(appID, VERSION, argc, argv,
+				   yahtzee_options, 0, NULL);
+
+	if (!NumberOfComputers)
+	  NumberOfComputers = 
+	    gnome_config_get_int("/gyahtzee/Preferences/NumberOfComputerOpponents=5");
+	if (!NumberOfHumans)
+	  NumberOfHumans = 
+	    gnome_config_get_int("/gyahtzee/Preferences/NumberOfHumanOpponents=1");
 
 	/* For some options, we don't want to play a game */
 	if (!GyahtzeeAbort) {
