@@ -27,13 +27,6 @@
 
 #include "button-images.h"
 
-/* If defined this does a very bad job of rendering
- * sequence numbers on top of the tiles as they are drawn.
- */
-/* #define SEQUENCE_DEBUG */
-
-#define GAME_EVENTS (GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK)
-
 typeinfo type_info [MAX_TILES+70] = {
   	{ 0, 0, {0, 0} },
 	{ 0, 0, {0, 0} },
@@ -296,15 +289,19 @@ tilepos hard_map [MAX_TILES] = {
 	{ 12, 0,  0},
 	{ 14, 0,  0}
 };
+
 tilepos *pos = hard_map ;
+int items_created = 0;
+int xpos_offset;
+int ypos_offset;
+
 tile tiles[MAX_TILES];
 
 GtkWidget *window, *pref_dialog, *hint_dialog;
 GtkWidget *mbox;
-GtkWidget *draw_area;
+GtkWidget *canvas;
 GtkWidget *tiles_label;
 GdkPixmap *tiles_pix, *mask;
-GdkGC *my_gc;
 int selected_tile, visible_tiles;
 int sequence_number;
 
@@ -326,8 +323,8 @@ static struct {
 static struct {
   GdkColor colour ;
   char *name ;
-  int set ;
-} backgnd = {0,0,0} ;
+  int set;
+} backgnd = {{0,0,0,0},NULL,0} ;
 
 struct _maps
 {
@@ -337,6 +334,14 @@ struct _maps
 } maps[] = { { "easy",      easy_map },
 	     { "difficult", hard_map } } ;
 
+gint hint_tiles[2];
+guint timer;
+guint timeout_counter = 0;
+
+void clear_undo_queue ();
+void you_won (void);
+void no_match (void);
+void check_free (void);
 void set_map (char *name) ;
 void load_tiles (char *fname);
 void quit_game_callback (GtkWidget *widget, gpointer data);
@@ -348,49 +353,47 @@ void select_game_callback (GtkWidget *widget, gpointer date);
 void new_game (void);
 void hint_callback (GtkWidget *widget, gpointer data);
 void properties_callback (GtkWidget *widget, gpointer data);
-void redraw_area (int x1, int y1, int x2, int y2, int mlayer);
-void redraw_tile (int i);
 void about_callback (GtkWidget *widget, gpointer data);
 void show_tb_callback (GtkWidget *widget, gpointer data);
 void sound_on_callback (GtkWidget *widget, gpointer data);
 
 
 GnomeUIInfo filemenu [] = {
-         {GNOME_APP_UI_ITEM, N_("New"), NULL, new_game_callback, NULL, NULL,
+         {GNOME_APP_UI_ITEM, N_("_New"), NULL, new_game_callback, NULL, NULL,
+         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 'n', GDK_CONTROL_MASK, NULL},
+
+         {GNOME_APP_UI_ITEM, N_("New _Seed..."), NULL, select_game_callback, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 0, 0, NULL},
 
-         {GNOME_APP_UI_ITEM, N_("New Seed..."), NULL, select_game_callback, NULL, NULL,
-         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 0, 0, NULL},
-
-         {GNOME_APP_UI_ITEM, N_("Restart"), NULL, restart_game_callback, NULL, NULL,
+         {GNOME_APP_UI_ITEM, N_("_Restart"), NULL, restart_game_callback, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REFRESH, 0, 0, NULL},
 
          {GNOME_APP_UI_SEPARATOR},
 
-         {GNOME_APP_UI_ITEM, N_("Hint"), NULL, hint_callback, NULL, NULL,
+         {GNOME_APP_UI_ITEM, N_("_Hint"), NULL, hint_callback, NULL, NULL,
          GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
-         {GNOME_APP_UI_ITEM, N_("Undo"), NULL, undo_tile_callback, NULL, NULL,
-         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO, 0, 0, NULL},
+         {GNOME_APP_UI_ITEM, N_("_Undo"), NULL, undo_tile_callback, NULL, NULL,
+         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO, 'z', GDK_CONTROL_MASK, NULL},
 
          {GNOME_APP_UI_SEPARATOR},
 	
-         {GNOME_APP_UI_ITEM, N_("Exit"), NULL, quit_game_callback, NULL, NULL,
-         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 0, 0, NULL},
+         {GNOME_APP_UI_ITEM, N_("E_xit"), NULL, quit_game_callback, NULL, NULL,
+         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'q', GDK_CONTROL_MASK, NULL},
 
          {GNOME_APP_UI_ENDOFINFO}
 };
 
 GnomeUIInfo optionsmenu [] = {
-	{GNOME_APP_UI_TOGGLEITEM, N_("Show Tool Bar"), NULL, show_tb_callback, NULL, NULL,
+	{GNOME_APP_UI_TOGGLEITEM, N_("Show _Tool Bar"), NULL, show_tb_callback, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
         {GNOME_APP_UI_SEPARATOR},
 
-        {GNOME_APP_UI_TOGGLEITEM, N_("Sound"), NULL, NULL, NULL, NULL,
+        {GNOME_APP_UI_TOGGLEITEM, N_("_Sound"), NULL, NULL, NULL, NULL,
         GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
-        {GNOME_APP_UI_ITEM, N_("Properties..."), NULL, properties_callback, NULL, NULL,
+        {GNOME_APP_UI_ITEM, N_("_Properties..."), NULL, properties_callback, NULL, NULL,
         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, 0, NULL},
 
         
@@ -398,33 +401,33 @@ GnomeUIInfo optionsmenu [] = {
 };
 
 GnomeUIInfo helpmenu[] = {
-	{GNOME_APP_UI_HELP, NULL, NULL, NULL, NULL, NULL,
-        GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+/* 	{GNOME_APP_UI_HELP, NULL, NULL, NULL, NULL, NULL, */
+/*         GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL}, */
 	
-	{GNOME_APP_UI_ITEM, N_("About"), NULL, about_callback, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("_About Mahjongg"), NULL, about_callback, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL},
 	
 	{GNOME_APP_UI_ENDOFINFO}
 };
 
 GnomeUIInfo mainmenu [] = {
-	{GNOME_APP_UI_SUBTREE, N_("Game"), NULL, filemenu, NULL, NULL,
+	{GNOME_APP_UI_SUBTREE, N_("_Game"), NULL, filemenu, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
-         {GNOME_APP_UI_SUBTREE, N_("Options"), NULL, optionsmenu, NULL, NULL,
+         {GNOME_APP_UI_SUBTREE, N_("_Options"), NULL, optionsmenu, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 	
-	{GNOME_APP_UI_SUBTREE, N_("Help"), NULL, helpmenu, NULL, NULL,
+	{GNOME_APP_UI_SUBTREE, N_("_Help"), NULL, helpmenu, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 	
 	{GNOME_APP_UI_ENDOFINFO}
 };
 
-GnomeUIInfo toolbar [] = {
-         {GNOME_APP_UI_ITEM, N_("New"), NULL, new_game_callback, NULL, NULL,
+GnomeUIInfo toolbar_uiinfo [] = {
+	{GNOME_APP_UI_ITEM, N_("New"), NULL, new_game_callback, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_NEW, 0, 0, NULL},
 
-         {GNOME_APP_UI_ITEM, N_("New Seed"), NULL, select_game_callback, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("Seed"), NULL, select_game_callback, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_NEW, 0, 0, NULL},
 
          {GNOME_APP_UI_ITEM, N_("Restart"), NULL, restart_game_callback, NULL, NULL,
@@ -442,7 +445,7 @@ GnomeUIInfo toolbar [] = {
          {GNOME_APP_UI_TOGGLEITEM, N_("Sound"), NULL, sound_on_callback, NULL, NULL,
          GNOME_APP_PIXMAP_DATA, mini_sound_xpm, 0, 0, NULL},
 
-         {GNOME_APP_UI_ITEM, N_("Properties"), NULL, properties_callback, NULL, NULL,
+         {GNOME_APP_UI_ITEM, N_("Prop."), NULL, properties_callback, NULL, NULL,
          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PROPERTIES, 0, 0, NULL},
 
          {GNOME_APP_UI_SEPARATOR},
@@ -465,7 +468,7 @@ hint_cancel (GtkWidget *widget, void *data)
     if (tiles[lp].selected == 17)
       {
 	tiles[lp].selected = 0 ;
-	redraw_tile(lp) ;
+/* 	redraw_tile(lp) ; */
       }
 
   gtk_widget_destroy (hint_dialog);
@@ -503,21 +506,11 @@ free_str (GtkWidget *widget, void *data)
 }
 
 
-void clear_window ()
-{
-  int x, y, w, h, d ;
-  gdk_window_get_geometry (draw_area->window, &x, &y, &w, &h, &d) ;
-  gdk_window_clear_area(draw_area->window, 0, 0, 1000, 1000) ;
-}
 
 void set_backgnd_colour (char *str)
 {
-  /* backgnd.colour.red = 0;
-  backgnd.colour.green = 0x3040;
-  backgnd.colour.blue = 0x1030;
-  backgnd.colour.pixel = 0; */
-
   GdkColormap *colourmap ;
+  GtkStyle *widget_style, *temp_style;
 
   if (str != backgnd.name)
     {
@@ -525,14 +518,88 @@ void set_backgnd_colour (char *str)
 	free (backgnd.name) ;
       backgnd.name = strdup (str) ;
     }
-  colourmap = gtk_widget_get_colormap(draw_area) ;
-  gdk_color_parse (backgnd.name, &backgnd.colour) ;
+  colourmap = gtk_widget_get_colormap(canvas);
+  gdk_color_parse (backgnd.name, &backgnd.colour);
 
-  gdk_color_alloc(colourmap, &backgnd.colour) ;
+  gdk_color_alloc(colourmap, &backgnd.colour);
 
-  gdk_window_set_background (draw_area->window, &backgnd.colour);
-  clear_window() ;
-  gtk_widget_draw (draw_area, NULL);
+  widget_style = gtk_widget_get_style (canvas);
+  temp_style = gtk_style_copy (widget_style);
+  temp_style->bg[0] = backgnd.colour;
+  temp_style->bg[1] = backgnd.colour;
+  temp_style->bg[2] = backgnd.colour;
+  temp_style->bg[3] = backgnd.colour;
+  temp_style->bg[4] = backgnd.colour;
+  gtk_widget_set_style (canvas, temp_style);
+  gnome_canvas_update_now (GNOME_CANVAS(canvas));
+}
+
+static void
+change_tile_image (tile *tile_inf) {
+	GdkImlibImage *new_image;
+	gint orig_x, orig_y;
+
+	orig_x = (tile_inf->image % 21) * TILE_WIDTH;
+	orig_y = (tile_inf->image / 21) * TILE_HEIGHT;
+
+	if ((tile_inf->selected == 1) || (tile_inf->selected == 17)) {
+		orig_y += 2 * TILE_HEIGHT;
+	}
+
+	gdk_imlib_destroy_image (tile_inf->current_image);
+	
+	tile_inf->current_image = new_image = gdk_imlib_crop_and_clone_image (tiles_image, orig_x, orig_y, TILE_WIDTH, TILE_HEIGHT);
+
+	gnome_canvas_item_set (tile_inf->image_item, "image", new_image, NULL);
+}
+
+static void
+tile_event (GnomeCanvasItem *item, GdkEvent *event, tile *tile_inf)
+{
+  gchar tmpchar[4];
+  
+  switch(event->type) {
+	  case GDK_BUTTON_PRESS :
+	    if((event->button.button == 1) && tile_free(tile_inf->number)) {
+	      if (tile_inf->selected == 1) {
+		selected_tile = MAX_TILES + 1;
+		tile_inf->selected = 0;
+		change_tile_image (tile_inf);
+	      }
+	      else {
+		if (selected_tile < MAX_TILES) {
+		  if ((tiles[selected_tile].type == tile_inf->type) && tile_free (tile_inf->number)) {
+		    tiles[selected_tile].visible = 0;
+		    tile_inf->visible = 0;
+		    tiles[selected_tile].selected = 0;
+		    change_tile_image (&tiles[selected_tile]);
+		    gnome_canvas_item_hide (tiles[selected_tile].image_item);
+		    gnome_canvas_item_hide (tile_inf->image_item);
+		    clear_undo_queue ();
+		    tiles[selected_tile].sequence = tile_inf->sequence = sequence_number;
+		    sequence_number ++;
+		    selected_tile = MAX_TILES + 1;
+		    visible_tiles -= 2;
+		    sprintf(tmpchar, "%d", visible_tiles);
+		    gtk_label_set (GTK_LABEL(tiles_label), tmpchar);
+		    check_free();
+		    if (visible_tiles <= 0)
+		      you_won ();
+		  }
+		  else
+		    no_match ();
+		}
+		else if (tile_free(tile_inf->number)) {
+		  tile_inf->selected = 1;
+		  change_tile_image(tile_inf);
+		  selected_tile = tile_inf->number;
+		}
+	      }
+	    }
+	    break;
+	  default :
+	    break;
+  }
 }
 
 static void
@@ -577,7 +644,7 @@ load_callback (GtkWidget *widget, void *data)
     printf ("Synced\n") ;
     }
   if (redraw)
-    gtk_widget_draw (draw_area, NULL);
+    gnome_canvas_update_now(GNOME_CANVAS(canvas));
   pref_cancel (0,0);
 }
 
@@ -680,13 +747,6 @@ int find_tile (int x, int y)
 	return tile_num;
 }
 
-void draw_selected_tile (int i)
-{
-	redraw_area (tiles[i].x, tiles[i].y,
-		     tiles[i].x + TILE_WIDTH - 1, tiles[i].y + TILE_HEIGHT - 1,
-		     -1);
-}
-
 void no_match (void)
 {
 	GtkWidget *mb;
@@ -695,7 +755,7 @@ void no_match (void)
 				   GNOME_MESSAGE_BOX_INFO,
 				   _("Ok"), NULL);
 	GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-	gnome_dialog_set_modal (GNOME_DIALOG (mb));
+	gtk_window_set_modal (&GNOME_MESSAGE_BOX(mb)->dialog.window, TRUE);
 	gtk_widget_show (mb);
 }
 
@@ -718,24 +778,11 @@ void check_free (void)
  					    GNOME_MESSAGE_BOX_INFO, 
  					    _("Ok"), NULL); 
  		GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE; 
- 		gnome_dialog_set_modal (GNOME_DIALOG (mb)); 
+		gtk_window_set_modal (&GNOME_MESSAGE_BOX(mb)->dialog.window, TRUE);
  		gtk_widget_show (mb); 
  	} 
 }
 
-void redraw_tile (int i)
-{
-#ifndef PLACE_DEBUG
-	gdk_window_clear_area (draw_area->window, tiles[i].x, tiles[i].y,
-			       TILE_WIDTH, TILE_HEIGHT);
-#endif
-
-	redraw_area (tiles[i].x, tiles[i].y, 
-		     tiles[i].x + TILE_WIDTH-1,
-		     tiles[i].y + TILE_HEIGHT-1,
-		     0);
-}
-
 void you_won (void)
 {
 	GtkWidget *mb;
@@ -744,7 +791,7 @@ void you_won (void)
 				   GNOME_MESSAGE_BOX_INFO,
 				   _("Ok"), NULL);
 	GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-	gnome_dialog_set_modal (GNOME_DIALOG (mb));
+	gtk_window_set_modal (GTK_WINDOW (mb), TRUE);
 	gtk_signal_connect_object (GTK_OBJECT(mb),
 				   "clicked",
 				   GTK_SIGNAL_FUNC (new_game_callback),
@@ -752,12 +799,13 @@ void you_won (void)
 	gtk_widget_show (mb);
 }
 
-void colour_changed_cb( GnomeColorSelector *widget, gchar **color )
+void colour_changed_cb (GtkWidget *w, int r1, int g1, int b1, int a1, 
+			gchar ** color)
 {
   static char tmp[24] = "" ;
-  int r,g,b;
+  guint8 r,g,b,a;
 
-  gnome_color_selector_get_color_int(widget, &r, &g, &b, 255 );
+  gnome_color_picker_get_i8(GNOME_COLOR_PICKER(w), &r, &g, &b, &a);
   
   sprintf( tmp, "#%02x%02x%02x", r, g, b ) ;
 
@@ -770,7 +818,7 @@ void properties_callback (GtkWidget *widget, gpointer data)
 	GtkDialog *d;
 	GtkWidget *button;
 	GtkWidget *tmenu, *mmenu, *otmenu, *ommenu, *l, *hb, *cb, *f, *fv;
-	GnomeColorSelector *bcolour_gcs ;
+	GtkWidget *bcolour_gcs ;
 
 	if (pref_dialog)
 		return;
@@ -853,16 +901,17 @@ void properties_callback (GtkWidget *widget, gpointer data)
 
 	{
 	  int ur,ug,ub ;
-	  bcolour_gcs  = gnome_color_selector_new( (SetColorFunc)colour_changed_cb,
-						   &backgnd.name) ;
+	  bcolour_gcs  = gnome_color_picker_new();
 	  sscanf( backgnd.name, "#%02x%02x%02x", &ur,&ug,&ub );
-	  gnome_color_selector_set_color_int( bcolour_gcs, ur, ug, ub, 255) ;
+	  gnome_color_picker_set_i8( GNOME_COLOR_PICKER(bcolour_gcs), ur, 
+				     ug, ub, 0);
+	  gtk_signal_connect(GTK_OBJECT(bcolour_gcs), "color_set", 
+			GTK_SIGNAL_FUNC(colour_changed_cb), &backgnd.name);
 	}
 
 	l = gtk_label_new (_("Background colour")) ;
 	gtk_box_pack_start_defaults (GTK_BOX(f), l) ;
-	gtk_box_pack_start_defaults (GTK_BOX(f),
-				     gnome_color_selector_get_button(bcolour_gcs)) ;
+	gtk_box_pack_start_defaults (GTK_BOX(f), bcolour_gcs);
 	gtk_widget_show(l) ;
 	gtk_box_pack_start_defaults (GTK_BOX(d->vbox), f) ;
 	gtk_widget_show(f) ;
@@ -884,10 +933,29 @@ void properties_callback (GtkWidget *widget, gpointer data)
 	
 }
 
+gint hint_timeout (gpointer data)
+{
+  if (timeout_counter <= 4) {
+    if (tiles[hint_tiles[0]].selected == 17) {
+      tiles[hint_tiles[0]].selected = 0;
+      tiles[hint_tiles[1]].selected = 0;
+    }
+    else {
+      tiles[hint_tiles[0]].selected = 17;
+      tiles[hint_tiles[1]].selected = 17;
+    }
+    change_tile_image(&tiles[hint_tiles[0]]);
+    change_tile_image(&tiles[hint_tiles[1]]);
+  }
+  else {
+    gtk_timeout_remove (timer);
+  }
+  timeout_counter ++;
+  return 1;
+}
+
 void hint_callback (GtkWidget *widget, gpointer data)
 {
-  GtkDialog *d;
-  GtkWidget *button, *f, *t;
   int i, j, free=0, type ;
 
   if (hint_dialog)
@@ -896,6 +964,12 @@ void hint_callback (GtkWidget *widget, gpointer data)
   /* Snarfed from check free
    * Tile Free is now _so_ much quicker, it is more elegant to do a
    * British Library search, and safer. */
+
+  if (selected_tile < MAX_TILES) {
+	  tiles[selected_tile].selected = 0;
+  }
+  selected_tile = MAX_TILES + 1;
+  
   for (i=0;i<MAX_TILES && !free;i++)
     if (tile_free(i))
       {
@@ -909,8 +983,10 @@ void hint_callback (GtkWidget *widget, gpointer data)
 		  tiles[i].selected = 17 ;
 		if (!tiles[j].selected)
 		  tiles[j].selected = 17 ;
-		redraw_tile(i) ;
-		redraw_tile(j) ;
+		change_tile_image (&tiles[i]);
+		change_tile_image (&tiles[j]);
+		hint_tiles[0] = i;
+		hint_tiles[1] = j;
 	      }
 	  }
       }
@@ -918,31 +994,9 @@ void hint_callback (GtkWidget *widget, gpointer data)
     for (i=0;i<MAX_TILES;i++)
     if (tiles[i].selected == 17)
     tiles[i].visible = 0 ;*/
-  
-  hint_dialog = gtk_dialog_new ();
-  d = GTK_DIALOG(hint_dialog);
-  gtk_signal_connect (GTK_OBJECT(hint_dialog), "delete_event",
-		      (GtkSignalFunc)hint_cancel, NULL); 
 
-  f = gtk_vbox_new (0, 5) ;
-  gtk_container_border_width (GTK_CONTAINER(f), 5) ;
-
-    t = gtk_label_new (_("Click to continue")) ;
-    gtk_box_pack_start_defaults (GTK_BOX(f), t) ;
-    gtk_widget_show (t) ;
-
-  gtk_box_pack_start_defaults (GTK_BOX(d->vbox), f) ;
-  gtk_widget_show (f) ;
-
-  /* Misc bottom buttons */
-  button = gnome_stock_button(GNOME_STOCK_BUTTON_OK);
-  gtk_signal_connect(GTK_OBJECT(button), "clicked", 
-		     GTK_SIGNAL_FUNC(hint_cancel), NULL); 
-  gtk_box_pack_start(GTK_BOX(d->action_area), button, TRUE, TRUE, 5);
-  gtk_widget_show(button);
-  
-  gtk_widget_show (hint_dialog);
-	
+  timeout_counter = 0;
+  timer = gtk_timeout_add (250, (GtkFunction) hint_timeout, NULL);
 }
 
 void about_callback (GtkWidget *widget, gpointer data)
@@ -991,10 +1045,11 @@ void restart_game_callback (GtkWidget *widget, gpointer data)
         tiles[i].visible = 1;
         tiles[i].selected = 0;
 	tiles[i].sequence = 0;
+	gnome_canvas_item_show (tiles[i].image_item);
     }
     selected_tile=MAX_TILES+1;
     sequence_number = 1 ;
-    gtk_widget_draw (draw_area, NULL);
+    gnome_canvas_update_now(GNOME_CANVAS(canvas));
 }
 
 void redo_tile_callback (GtkWidget *widget, gpointer data)
@@ -1008,7 +1063,7 @@ void redo_tile_callback (GtkWidget *widget, gpointer data)
     if (selected_tile<MAX_TILES) 
       {
 	tiles[selected_tile].selected = 0 ;
-	redraw_tile (selected_tile) ;
+	change_tile_image (&tiles[i]);
 	selected_tile = MAX_TILES + 1; 
       }
     change = 0 ;
@@ -1017,8 +1072,8 @@ void redo_tile_callback (GtkWidget *widget, gpointer data)
 	{
 	  tiles[i].selected = 0 ;
 	  tiles[i].visible = 0 ;
+	  gnome_canvas_item_hide (tiles[i].image_item);
 	  visible_tiles-- ;
-	  redraw_tile (i);
 	  change = 1 ;
 	}
     if (change)
@@ -1027,6 +1082,7 @@ void redo_tile_callback (GtkWidget *widget, gpointer data)
 
     sprintf(tmpchar,"%3d",visible_tiles) ;
     gtk_label_set(GTK_LABEL(tiles_label), tmpchar);
+    gnome_canvas_update_now (GNOME_CANVAS(canvas));
 }
 
 void undo_tile_callback (GtkWidget *widget, gpointer data)
@@ -1042,7 +1098,7 @@ void undo_tile_callback (GtkWidget *widget, gpointer data)
     if (selected_tile<MAX_TILES) 
       {
 	tiles[selected_tile].selected = 0 ;
-	redraw_tile (selected_tile) ;
+	change_tile_image (&tiles[i]);
 	selected_tile = MAX_TILES + 1; 
       }
 
@@ -1052,11 +1108,12 @@ void undo_tile_callback (GtkWidget *widget, gpointer data)
 	  tiles[i].selected = 0 ;
 	  tiles[i].visible = 1 ;
 	  visible_tiles++ ;
-	  redraw_tile(i) ;
+	  gnome_canvas_item_show (tiles[i].image_item);
 	}
 
     sprintf(tmpchar,"%d",visible_tiles) ;
     gtk_label_set(GTK_LABEL(tiles_label), tmpchar);
+    gnome_canvas_update_now (GNOME_CANVAS(canvas));
 }
 
 static void
@@ -1119,15 +1176,21 @@ void select_game_callback (GtkWidget *widget, gpointer data)
 
 void show_tb_callback (GtkWidget *widget, gpointer data)
 {
+    GnomeDockItem *gdi;
+    GtkWidget *toolbar;
+
+    gdi = gnome_app_get_dock_item_by_name (GNOME_APP (window), GNOME_APP_TOOLBAR_NAME);
+    toolbar = gnome_dock_item_get_child (gdi);
+
     if((GTK_CHECK_MENU_ITEM(optionsmenu[0].widget))->active)
     {
         gnome_config_set_bool("gmahjongg/toolbar/show", TRUE);
-        gtk_widget_show(GTK_WIDGET(GNOME_APP(window)->toolbar->parent));
+        gtk_widget_show(gdi);
     }
     else
     {
         gnome_config_set_bool("gmahjongg/toolbar/show", FALSE);
-        gtk_widget_hide(GTK_WIDGET(GNOME_APP(window)->toolbar->parent));
+        gtk_widget_hide(gdi);
     }
 }
 
@@ -1136,119 +1199,6 @@ void sound_on_callback (GtkWidget *widget, gpointer data)
     printf("mer\n");
 }
 
-void redraw_tile_in_area (int x1, int y1, int x2, int y2, int tile)
-{
-	GdkRectangle trect, arect, dest;
-	int orig_x, orig_y;
-
-#ifdef SEQUENCE_DEBUG
-	int n ;
-	GdkFont *font ;
-	char fontname[1024] ;
-	char tmpstr[4] ;
-	GdkColor color;
-	GdkGC *gc ;
-	GdkColorContext *cc;
-	sprintf(fontname, "-bitstream-courier-bold-r-*-*-%d-*-*-*-*-*-*-*", 16);	
-	font = gdk_font_load(fontname);
-	gc = gdk_gc_new(draw_area->window);
-	color.red   = 0 ;
-	color.green = 0 ;
-	color.blue  = 0 ;
-	color.pixel = 0; /* required! */
-	cc = gdk_color_context_new (gtk_widget_get_visual (draw_area),
-				    gtk_widget_get_colormap (draw_area));
-
-	n = 0;
-	gdk_color_context_get_pixels (cc, &color.red, &color.green, &color.blue, 1, &color.pixel, &n) ;
-	gdk_gc_set_foreground(gc, &color);
-#endif
-
-	if (tile < MAX_TILES) {
-		trect.x = tiles[tile].x;
-		trect.y = tiles[tile].y;
-		trect.width = TILE_WIDTH;
-		trect.height = TILE_HEIGHT;
-
-		arect.x = x1;
-		arect.y = y1;
-		arect.width = x2 - x1 + 1;
-		arect.height = y2 - y1 + 1;
-
-		if (gdk_rectangle_intersect (&trect, &arect, &dest)) {
-			/* 21 is the number of tiles in a row of the source image */
-
-			orig_x = (tiles[tile].image % 21) * TILE_WIDTH + dest.x - trect.x;
-			orig_y = (tiles[tile].image / 21) * TILE_HEIGHT + dest.y - trect.y;
-
-			if (tiles[tile].selected)
-				orig_y += 2 * TILE_HEIGHT;
-
-			gdk_gc_set_clip_origin (my_gc, trect.x, trect.y);
-
-			gdk_draw_pixmap (draw_area->window, 
-					 my_gc,
-					 tiles_pix,
-					 orig_x, orig_y,
-					 dest.x, dest.y,
-					 dest.width, dest.height);
-
-#ifdef SEQUENCE_DEBUG
-			sprintf (tmpstr, "%d", tiles[tile].sequence) ;
-			gdk_draw_string(draw_area->window,
-					font, gc,
-					dest.x+4, dest.y+16,
-					tmpstr) ;
-			sprintf (tmpstr, "%d", tile) ;
-			gdk_draw_string(draw_area->window,
-					font, gc,
-					dest.x+4, dest.y+32+16,
-					tmpstr) ;
-#endif
-		}
-	}
-}
-
-void redraw_area (int x1, int y1, int x2, int y2, int mlayer)
-{
-	int i;
-	GdkRectangle area, trect, dest;
-
-	area.x = x1;
-	area.y = y1;
-	area.width = x2 - x1 + 1;
-        area.height = y2 - y1 + 1;
-
-        for(i = MAX_TILES; i > -1; i --) {
-        	if ((tiles[i].visible) && (tiles[i].layer >= mlayer)) {
-			trect.x = tiles[i].x;
-			trect.y = tiles[i].y;
-			trect.width = TILE_WIDTH;
-			trect.height = TILE_HEIGHT;
-
-			if (gdk_rectangle_intersect (&area, &trect, &dest))
-				redraw_tile_in_area (x1, y1, x2, y2, i);
-		}
-	}
-}
-
-void refresh (GdkRectangle *area)
-{
-  redraw_area (area->x, area->y, area->x + area->width - 1,
-	       area->y + area->height - 1, 0);
-}
-
-void tile_gone (int i, int x, int y)
-{
-	gdk_window_clear_area (draw_area->window, tiles[i].x, tiles[i].y,
-			       TILE_WIDTH, TILE_HEIGHT);
-	
-	redraw_area (tiles[i].x, tiles[i].y, 
-		     tiles[i].x + TILE_WIDTH - 1,
-		     tiles[i].y + TILE_HEIGHT - 1,
-		     0);
-}
-
 /* You loose your re-do queue when you make a move */
 void clear_undo_queue ()
 {
@@ -1258,103 +1208,83 @@ void clear_undo_queue ()
       tiles[lp].sequence = 0 ;
 }
 
-void button_pressed (int x, int y)
-{
-	int i;
-        gchar tmpchar[4];
-        
-	i = find_tile (x, y);
-	if (i < MAX_TILES) {
-	    if (tile_free (i)) {
-/*		printf ("{ %d %d %d },\n",
-			(tiles[i].x - 5*tiles[i].layer - 30)/HALF_WIDTH,
-			(tiles[i].y + 4*tiles[i].layer - 25)/HALF_HEIGHT, tiles[i].layer) */
-		if (selected_tile < MAX_TILES) {
-			    if ((tiles[selected_tile].type == tiles[i].type) &&
-				    (i != selected_tile) ) {
-					tiles[i].visible = 0;
-					tiles[selected_tile].visible = 0;
-					tile_gone (i, x, y);
-					tile_gone (selected_tile,
-						   tiles[selected_tile].x + 1,
-						   tiles[selected_tile].y + 1);
-					clear_undo_queue() ;
-					tiles[i].sequence = sequence_number ;
-					tiles[selected_tile].sequence = sequence_number ;
-					sequence_number++ ;
-					selected_tile = MAX_TILES + 1;
-                                        visible_tiles -= 2;
-                                        sprintf(tmpchar,"%d",visible_tiles);
-                                        gtk_label_set(GTK_LABEL(tiles_label), tmpchar);
-					check_free();
-					if (visible_tiles <= 0)
-						you_won ();
-				}
-				else if (i == selected_tile) {
-					tiles[i].selected = 0;
-					selected_tile = MAX_TILES + 1;
-				}
-				else {
-					no_match();
-					i = selected_tile;
-				}
-			}
-			else if (tiles[i].selected == 0) {
-				tiles[i].selected = 1;
-				selected_tile = i;
-			}
-			else {
-				tiles[i].selected = 0;
-				selected_tile = MAX_TILES + 1;
-			}
-			draw_selected_tile (i);
-		}
-	}
-}
-
-
-gint area_event (GtkWidget *widget, GdkEvent *event, void *d)
-{
-	switch (event->type) {
-	case GDK_EXPOSE : {
-		GdkEventExpose *e = (GdkEventExpose *) event;
-		refresh (&e->area);
-		return TRUE;
-	}
-
-	case GDK_BUTTON_PRESS: {
-		int x, y;
-
-		gtk_widget_get_pointer (widget, &x, &y);
-		button_pressed (x, y);
-		return TRUE;
-	}
-
-	default:
-		return FALSE;
-	}
-}
-
 void set_map (char *name)
 {
   int lp ;
+  int xmax = 0, ymax = 0;
+  tilepos *t;
+  int i;
 
   for (lp=0;lp<sizeof(maps)/sizeof(struct _maps);lp++)
     if (strcasecmp (maps[lp].name, name) == 0)
       {
 	pos = maps[lp].map ;
-	generate_dependancies() ;
-	new_game () ;
 
+	for ( t = pos ; t < pos + MAX_TILES ; t++ ) {
+	    if ( (*t).x  > xmax ) xmax = (*t).x;
+	    if ( (*t).y  > ymax ) ymax = (*t).y;
+	}
+	xpos_offset = ( AREA_WIDTH - (HALF_WIDTH * (xmax+1)) ) / 2;
+	ypos_offset = ( AREA_HEIGHT - (HALF_HEIGHT * (ymax+1) ) ) / 2;
+
+	generate_dependancies() ;
 	if (mapset)
 	  g_free (mapset);
 	mapset = g_strdup(name);
       }
+  new_game ();
+  if (items_created) {
+	  for (i = MAX_TILES - 1; i >= 0; i --) {
+		  gnome_canvas_item_set (tiles[i].image_item,
+					 "x", (double)tiles[i].x,
+					 "y", (double)tiles[i].y,
+					 NULL);
+	  }
+  }
+}
+
+void create_canvas_items (void)
+{
+  GdkImlibImage *image;
+  gint orig_x, orig_y, i, layer, max_x, tile_was_set = 0;
+  
+  for (layer = 0; layer < 7; layer ++) {
+    for (i = MAX_TILES - 1; i >= 0; i --) {
+      if (tiles[i].layer == layer) {
+	tiles[i].canvas_item = gnome_canvas_item_new (gnome_canvas_root(GNOME_CANVAS(canvas)),
+						      gnome_canvas_group_get_type (),
+						      NULL);
+	
+	orig_x = (tiles[i].image % 21) * TILE_WIDTH;
+	orig_y = (tiles[i].image / 21) * TILE_HEIGHT;
+	
+	tiles[i].number = i;
+	
+	tiles[i].current_image = image = gdk_imlib_crop_and_clone_image (tiles_image, orig_x, orig_y, TILE_WIDTH, TILE_HEIGHT);
+	
+	tiles[i].image_item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (tiles[i].canvas_item),
+						      gnome_canvas_image_get_type(),
+						      "image", image,
+						      "x", (double)tiles[i].x,
+						      "y", (double)tiles[i].y,
+						      "width", (double)TILE_WIDTH,
+						      "height", (double)TILE_HEIGHT,
+						      "anchor", GTK_ANCHOR_NW,
+						      NULL);
+	
+	gtk_signal_connect (GTK_OBJECT (tiles[i].canvas_item), "event",
+			    (GtkSignalFunc) tile_event,
+			    &tiles[i]);
+      }
+    }
+  }
+  items_created = 1;
 }
 
 void load_tiles (char *fname)
 {
 	char *tmp, *fn;
+	int i;
 
 	tmp = g_copy_strings ("mahjongg/", fname, NULL);
 
@@ -1378,7 +1308,13 @@ void load_tiles (char *fname)
 	gdk_imlib_render (tiles_image, tiles_image->rgb_width, tiles_image->rgb_height);
 	
 	tiles_pix = gdk_imlib_move_image (tiles_image);
-	mask = gdk_imlib_move_mask (tiles_image);
+
+	if (items_created) {
+		for (i = 0; i < MAX_TILES; i++) {
+			change_tile_image (&tiles[i]);
+		}
+	}
+		
 
 	g_free (fn);
 }
@@ -1391,39 +1327,31 @@ void create_mahjongg_board (void)
 	gtk_widget_push_visual (gdk_imlib_get_visual ());
 	gtk_widget_push_colormap (gdk_imlib_get_colormap ());
 
-	draw_area = gtk_drawing_area_new ();
+	canvas = gnome_canvas_new();
 
 	gtk_widget_pop_colormap ();
 	gtk_widget_pop_visual ();
 	
-	gtk_widget_set_events (draw_area, gtk_widget_get_events (draw_area) | GAME_EVENTS);
+	gtk_box_pack_start_defaults (GTK_BOX (mbox), canvas);
 
-	style = gtk_widget_get_style (draw_area);
-
-	gtk_box_pack_start_defaults (GTK_BOX (mbox), draw_area);
-	gtk_widget_realize (draw_area);
-
-	gtk_widget_show (draw_area);
-
-	gtk_drawing_area_size (GTK_DRAWING_AREA (draw_area),
-			       600,
-			       480);
-
-	buf = gnome_config_get_string_with_default ("/gmahjongg/Preferences/tileset=default.png", NULL);
-	load_tiles (buf);
+	gtk_widget_set_usize(canvas, AREA_WIDTH, AREA_HEIGHT);
+	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS(canvas), 1);
+	gnome_canvas_set_scroll_region(GNOME_CANVAS(canvas), 0, 0, AREA_WIDTH, AREA_HEIGHT);
+	
+	gtk_widget_show (canvas);
 
 	buf = gnome_config_get_string_with_default ("/gmahjongg/Preferences/mapset=easy", NULL);
 	set_map (buf);
 
+	buf = gnome_config_get_string_with_default ("/gmahjongg/Preferences/tileset=default.png", NULL);
+	load_tiles (buf);
+	new_game ();
+	create_canvas_items ();
+
 	buf = gnome_config_get_string_with_default ("/gmahjongg/Preferences/bcolour=#003010", NULL) ;
 	set_backgnd_colour (buf) ;
 
-	my_gc = gdk_gc_new (draw_area->window);
-	gdk_gc_set_clip_mask (my_gc, mask);
-
-	gtk_signal_connect (GTK_OBJECT (draw_area), "event", (GtkSignalFunc) area_event, 0);
-	
-	gtk_widget_draw (draw_area, NULL);
+	gnome_canvas_update_now(GNOME_CANVAS(canvas));
 	g_free (buf);
 	new_game ();
 }
@@ -1432,56 +1360,64 @@ void create_mahjongg_board (void)
 
 void new_game ()
 {
+  int i;
+  
   sequence_number = 1 ;
   visible_tiles = MAX_TILES;
   selected_tile = MAX_TILES + 1;
 
-  clear_window() ;
   generate_game() ;
-  gtk_widget_draw (draw_area, NULL) ;
+
+  if (items_created) {
+    for (i = 0; i < MAX_TILES; i++) {
+      change_tile_image (&tiles[i]);
+      gnome_canvas_item_show (tiles[i].image_item);
+    }
+  }
+  
+  gnome_canvas_update_now (GNOME_CANVAS(canvas));
 }
 
 int main (int argc, char *argv [])
 {
-	argp_program_version = MAH_VERSION;
-
+        GnomeDockItem *gdi;
+	GtkWidget *toolbar;
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
-	gnome_init ("mahjongg", NULL, argc, argv, 0, NULL);
+	gnome_init ("mahjongg", MAH_VERSION, argc, argv);
 
 	srand (time (NULL));
 	
 	generate_dependancies () ;
 
 	window = gnome_app_new ("gmahjongg", _("Gnome Mahjongg"));
-	gtk_widget_realize (window);
 	gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, TRUE);
 
 	gnome_app_create_menus (GNOME_APP (window), mainmenu);
-	gtk_menu_item_right_justify (GTK_MENU_ITEM(mainmenu[2].widget));
-
-        gnome_app_create_toolbar (GNOME_APP (window), toolbar);
-        gtk_toolbar_set_space_size(GTK_TOOLBAR(GNOME_APP(window)->toolbar), 25);
+        gnome_app_create_toolbar (GNOME_APP (window), toolbar_uiinfo);
+	gdi = gnome_app_get_dock_item_by_name (GNOME_APP (window), GNOME_APP_TOOLBAR_NAME);
+	toolbar = gnome_dock_item_get_child (gdi);
+        gtk_toolbar_set_space_size(GTK_TOOLBAR (toolbar), 25);
         
         if(gnome_config_get_bool("/gmahjongg/toolbar/show=TRUE"))
             gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(optionsmenu[0].widget), TRUE);
         else
-            gtk_widget_hide(GNOME_APP(window)->toolbar->parent);
+	    gtk_widget_hide(gdi);
 
-        tiles_label = gtk_label_new(_(" Tiles "));
+        tiles_label = gtk_label_new(_("Tiles"));
         gtk_widget_show(tiles_label);
-        gtk_toolbar_append_widget(GTK_TOOLBAR(GNOME_APP(window)->toolbar), tiles_label,
-                                  NULL, NULL);
-        tiles_label = gtk_label_new(_(" Remaining: "));
+	gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar), tiles_label,
+				  NULL, NULL);
+        tiles_label = gtk_label_new(_(" Left: "));
         gtk_widget_show(tiles_label);
-        gtk_toolbar_append_widget(GTK_TOOLBAR(GNOME_APP(window)->toolbar), tiles_label,
-                                  NULL, NULL);
+	gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar), tiles_label,
+				  NULL, NULL);
         tiles_label = gtk_label_new(MAX_TILES_STR);
         gtk_widget_show(tiles_label);
-        gtk_toolbar_append_widget(GTK_TOOLBAR(GNOME_APP(window)->toolbar), tiles_label,
-                                  NULL, NULL);
-        gtk_toolbar_append_space(GTK_TOOLBAR(GNOME_APP(window)->toolbar));
+	gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar), tiles_label,
+				  NULL, NULL);
+	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
 	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
 			    GTK_SIGNAL_FUNC (quit_game_callback), NULL);
