@@ -20,8 +20,6 @@
 #include <gdk_imlib.h>
 
 
-static error_t parse_an_arg (int key, char *arg, struct argp_state *state);
-
 #define STONE_SIZE 40
 #define STONE_COLS  15
 #define STONE_LINES 10
@@ -31,27 +29,6 @@ static error_t parse_an_arg (int key, char *arg, struct argp_state *state);
 		     GDK_LEAVE_NOTIFY_MASK          |\
 		     GDK_POINTER_MOTION_MASK)
 
-/* The command-line options we understand.  */
-static struct argp_option options[] =
-{
-  { "debug", 'd', NULL, 0, N_("Enable debugging"), 1 },
-  { "delete-sess", 'D', N_("ID"), OPTION_HIDDEN, NULL, 1 },
-  { "scenario", 's', N_("NAME"), 0, N_("Set game scenario"), 1 },
-  { NULL, 0, NULL, 0, NULL, 0 }
-};
-
-/* How to parse the command-line options.  */
-static struct argp parser =
-{
-  options,
-  parse_an_arg,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL
-};
-
 static GtkWidget *pref_dialog, *scorew;
 static GtkWidget *app, *draw_area, *vb;
 static GdkImlibImage *image;
@@ -269,7 +246,7 @@ set_score (int new_score)
 	char b [20];
 	
 	score = new_score;
-	sprintf (b, "%5d", score);
+	sprintf (b, "%.5d", score);
 	gtk_label_set (GTK_LABEL(scorew), b);
 }
 
@@ -647,6 +624,7 @@ game_about_callback (GtkWidget *widget, void *data)
 				 _("Original idea from KDE's same game program."),
 				 /*"gnome-same-gnome.xpm"*/
 				 NULL);
+	gnome_dialog_set_parent(GNOME_DIALOG(about), GTK_WINDOW(app));
 	gtk_widget_show (about);
 
 	return TRUE;
@@ -671,8 +649,9 @@ game_quit_callback (GtkWidget *widget, void *data)
 				     GNOME_STOCK_BUTTON_YES,
 				     GNOME_STOCK_BUTTON_NO,
 				     NULL);
-	gnome_dialog_set_default (GNOME_MESSAGE_BOX (box), 0);
-	gnome_dialog_set_modal (GNOME_MESSAGE_BOX (box));
+	gnome_dialog_set_parent (GNOME_DIALOG(box), GTK_WINDOW(app));
+	gnome_dialog_set_default (GNOME_DIALOG (box), 0);
+	gtk_window_set_modal (GTK_WINDOW (box), TRUE);
 	gtk_signal_connect (GTK_OBJECT (box), "clicked",
 			   (GtkSignalFunc)game_maybe_quit, NULL);
 	gtk_widget_show (box);
@@ -687,10 +666,10 @@ GnomeUIInfo gamemenu[] = {
 	{GNOME_APP_UI_ITEM, N_("_Properties..."), NULL, game_preferences_callback, NULL, NULL,
 	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, 0, NULL},
 
-	{GNOME_APP_UI_ITEM, N_("_Scores..."), NULL, game_top_ten_callback, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("_Scores"), NULL, game_top_ten_callback, NULL, NULL,
 	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SCORES, 0, 0, NULL},
 
-	{GNOME_APP_UI_ITEM, N_("_Quit"), NULL, game_quit_callback, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("E_xit"), NULL, game_quit_callback, NULL, NULL,
 	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'q', GDK_CONTROL_MASK, NULL},
 
 	{GNOME_APP_UI_ENDOFINFO}
@@ -700,7 +679,7 @@ GnomeUIInfo helpmenu[] = {
 	{GNOME_APP_UI_HELP, NULL, NULL, "samegnome", NULL, NULL,
 	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
-	{GNOME_APP_UI_ITEM, N_("_About..."), NULL, game_about_callback, NULL, NULL,
+	{GNOME_APP_UI_ITEM, N_("_About Same Gnome"), NULL, game_about_callback, NULL, NULL,
 	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL},
 
 	{GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL, NULL, NULL,
@@ -849,60 +828,57 @@ restart (gchar *id)
 	g_free(sess);
 }
 
-static void
-client_connect (GnomeClient *client, gint was_restarted)
+static gint
+client_die (GnomeClient *client, gpointer client_data)
 {
-	if (was_restarted) {
-		restarted = 1;
-		restart (gnome_client_get_previous_id (client));
-	}
+        gtk_exit (0);
+
+	return FALSE;
 }
 
-static error_t
-parse_an_arg (int key, char *arg, struct argp_state *state)
-{
-	switch(key) {
-	case 'd':
-		debugging = 1;
-		g_print ("Debugging mode\n");
-		break;
-	case 'D':
-		delete_session (arg);
-		exit(0);
-		break;
-	case 's':
-		if (fname)
-			argp_usage (state);
-		fname = g_strdup (arg);
-		break;
-	default:
-		return ARGP_ERR_UNKNOWN;
-	}
+static char *session_id = NULL;
 
-	return 0;
-}
+static const struct poptOption options[] = {
+  {NULL, 'd', POPT_ARG_NONE, &debugging, 0, N_("Debugging mode"), NULL},
+  {"discard-session", 'D', POPT_ARG_STRING, &session_id, 0, N_("Discard session"), N_("ID")},
+  {"scenario", 's', POPT_ARG_STRING, &fname, 0, N_("Set game scenario"), N_("NAME")},
+  {NULL, '\0', 0, NULL, 0}
+};
 
 int
 main (int argc, char *argv [])
 {
-	GtkWidget *label, *hb;
+	GtkWidget *label, *hb, *frame;
 	GnomeClient *client;
 
-	argp_program_version = VERSION;
-
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
 	gnome_score_init("same-gnome");
 
-	client = gnome_client_new_default ();
+	gnome_init_with_popt_table ("same-gnome", VERSION, argc, argv, options, 0, NULL);
+	if(session_id) {
+	  delete_session (session_id);
+	  exit(0);
+	}
+
+	client= gnome_master_client ();
+
 	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
 			    GTK_SIGNAL_FUNC (save_state), argv[0]);
-	gtk_signal_connect (GTK_OBJECT (client), "connect",
-			    GTK_SIGNAL_FUNC (client_connect), NULL);
+	gtk_signal_connect (GTK_OBJECT (client), "die",
+			    GTK_SIGNAL_FUNC (client_die), NULL);
 
-	gnome_init ("same-gnome", &parser, argc, argv, 0, NULL);
-	gdk_imlib_init ();
+	if (GNOME_CLIENT_CONNECTED (client))
+	  {
+	    GnomeClient *cloned= gnome_cloned_client ();
+	    
+	    if (cloned)
+	      {
+		restarted= 1;
+		restart (gnome_client_get_id (cloned));
+	      }
+	  }
 
 	srand (time (NULL));
 
@@ -911,21 +887,29 @@ main (int argc, char *argv [])
 	gtk_signal_connect (GTK_OBJECT(app), "delete_event",
 			    (GtkSignalFunc)game_quit_callback, NULL);
 	gnome_app_create_menus(GNOME_APP(app), mainmenu);
-        gtk_menu_item_right_justify(GTK_MENU_ITEM(mainmenu[1].widget));
   
         vb = gtk_vbox_new (FALSE, 0);
 	hb = gtk_hbox_new (FALSE, 0);
+	gtk_container_border_width(GTK_CONTAINER(hb), 3);
 	gnome_app_set_contents (GNOME_APP (app), vb);
 
 	if (!fname)
 		fname = g_strdup ("stones.png");
 	create_same_board (fname);
 
+	frame = gtk_frame_new(NULL);
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
+	gtk_container_border_width(GTK_CONTAINER(frame), 0);
+	gtk_widget_show(frame);
+
 	label = gtk_label_new (_("Score: "));
 	scorew = gtk_label_new ("");
 	set_score (score);
-	gtk_box_pack_start_defaults (GTK_BOX(vb), hb);
-	gtk_box_pack_end (GTK_BOX(hb), scorew, 0, 0, 10);
+
+	gtk_container_add(GTK_CONTAINER(frame), hb);
+
+	gtk_box_pack_start(GTK_BOX(vb), frame, 1, 1, 0);
+	gtk_box_pack_end (GTK_BOX(hb), scorew, 0, 0, 0);
 	gtk_box_pack_end (GTK_BOX(hb), label,  0, 0, 0);
 	
 	if (!restarted)
@@ -933,11 +917,11 @@ main (int argc, char *argv [])
 	
 	g_free (fname);
 
-        gtk_widget_show (app);
 	gtk_widget_show (hb);
 	gtk_widget_show (vb);
 	gtk_widget_show (GTK_WIDGET(label));
 	gtk_widget_show (GTK_WIDGET(scorew));
+        gtk_widget_show (app);
 
 	gtk_main ();
 	return 0;
